import { LocationRange } from "peggy";
export declare const infixFunctions: {
    "+": string;
    "-": string;
    "!=": string;
    ".-": string;
    ".*": string;
    "./": string;
    ".^": string;
    ".+": string;
    "*": string;
    "/": string;
    "&&": string;
    "^": string;
    "<": string;
    "<=": string;
    "==": string;
    ">": string;
    ">=": string;
    "||": string;
    to: string;
};
export type InfixOperator = keyof typeof infixFunctions;
export declare const unaryFunctions: {
    "-": string;
    "!": string;
    ".-": string;
};
export type UnaryOperator = keyof typeof unaryFunctions;
type Node = {
    location: LocationRange;
};
type N<T extends string, V extends object> = Node & {
    type: T;
} & V;
type NodeBlock = N<"Block", {
    statements: ASTNode[];
}>;
type NodeProgram = N<"Program", {
    imports: [NodeString, NodeIdentifier][];
    statements: ASTNode[];
    symbols: {
        [k in string]: ASTNode;
    };
}>;
type NodeArray = N<"Array", {
    elements: ASTNode[];
}>;
type NodeDict = N<"Dict", {
    elements: AnyNodeDictEntry[];
    symbols: {
        [k in number | string]: AnyNodeDictEntry;
    };
}>;
type NodeKeyValue = N<"KeyValue", {
    key: ASTNode;
    value: ASTNode;
}>;
type AnyNodeDictEntry = NodeKeyValue | NodeIdentifier;
type NodeUnitValue = N<"UnitValue", {
    value: ASTNode;
    unit: string;
}>;
type NodeCall = N<"Call", {
    fn: ASTNode;
    args: ASTNode[];
}>;
type NodeInfixCall = N<"InfixCall", {
    op: InfixOperator;
    args: [ASTNode, ASTNode];
}>;
type NodeUnaryCall = N<"UnaryCall", {
    op: UnaryOperator;
    arg: ASTNode;
}>;
type NodePipe = N<"Pipe", {
    leftArg: ASTNode;
    fn: ASTNode;
    rightArgs: ASTNode[];
}>;
type NodeDotLookup = N<"DotLookup", {
    arg: ASTNode;
    key: string;
}>;
type NodeBracketLookup = N<"BracketLookup", {
    arg: ASTNode;
    key: ASTNode;
}>;
type NodeFloat = N<"Float", {
    integer: number;
    fractional: string | null;
    exponent: number | null;
}>;
type NodeIdentifierWithAnnotation = N<"IdentifierWithAnnotation", {
    variable: string;
    annotation: ASTNode;
}>;
type NodeIdentifier = N<"Identifier", {
    value: string;
}>;
type NodeLetStatement = N<"LetStatement", {
    variable: NodeIdentifier;
    value: ASTNode;
    exported: boolean;
}>;
type NodeLambda = N<"Lambda", {
    args: ASTNode[];
    body: ASTNode;
    name?: string;
}>;
type NamedNodeLambda = NodeLambda & Required<Pick<NodeLambda, "name">>;
type NodeDefunStatement = N<"DefunStatement", {
    variable: NodeIdentifier;
    value: NamedNodeLambda;
    exported: boolean;
}>;
type NodeTernary = N<"Ternary", {
    condition: ASTNode;
    trueExpression: ASTNode;
    falseExpression: ASTNode;
    kind: "IfThenElse" | "C";
}>;
type NodeString = N<"String", {
    value: string;
}>;
type NodeBoolean = N<"Boolean", {
    value: boolean;
}>;
type NodeVoid = N<"Void", object>;
export type ASTNode = NodeArray | NodeDict | NodeBlock | NodeProgram | NodeUnitValue | NodeCall | NodeInfixCall | NodeUnaryCall | NodePipe | NodeDotLookup | NodeBracketLookup | NodeFloat | NodeIdentifier | NodeIdentifierWithAnnotation | NodeLetStatement | NodeDefunStatement | NodeLambda | NodeTernary | NodeKeyValue | NodeString | NodeBoolean | NodeVoid;
export declare function nodeCall(fn: ASTNode, args: ASTNode[], location: LocationRange): NodeCall;
export declare function makeInfixChain(head: ASTNode, tail: [InfixOperator, ASTNode][], location: LocationRange): ASTNode;
export declare function nodeInfixCall(op: InfixOperator, arg1: ASTNode, arg2: ASTNode, location: LocationRange): NodeInfixCall;
export declare function nodeUnaryCall(op: UnaryOperator, arg: ASTNode, location: LocationRange): NodeUnaryCall;
export declare function nodePipe(leftArg: ASTNode, fn: ASTNode, rightArgs: ASTNode[], location: LocationRange): NodePipe;
export declare function nodeDotLookup(arg: ASTNode, key: string, location: LocationRange): NodeDotLookup;
export declare function nodeBracketLookup(arg: ASTNode, key: ASTNode, location: LocationRange): NodeBracketLookup;
export declare function nodeArray(elements: ASTNode[], location: LocationRange): NodeArray;
export declare function nodeDict(elements: AnyNodeDictEntry[], location: LocationRange): NodeDict;
export declare function nodeUnitValue(value: ASTNode, unit: string, location: LocationRange): NodeUnitValue;
export declare function nodeBlock(statements: ASTNode[], location: LocationRange): NodeBlock;
export declare function nodeProgram(imports: [NodeString, NodeIdentifier][], statements: ASTNode[], location: LocationRange): NodeProgram;
export declare function nodeBoolean(value: boolean, location: LocationRange): NodeBoolean;
export declare function nodeFloat(args: Omit<NodeFloat, "type" | "location">, location: LocationRange): NodeFloat;
export declare function nodeIdentifier(value: string, location: LocationRange): NodeIdentifier;
export declare function nodeIdentifierWithAnnotation(variable: string, annotation: ASTNode, location: LocationRange): NodeIdentifierWithAnnotation;
export declare function nodeKeyValue(key: ASTNode, value: ASTNode, location: LocationRange): NodeKeyValue;
export declare function nodeLambda(args: ASTNode[], body: ASTNode, location: LocationRange, name?: NodeIdentifier): NodeLambda;
export declare function nodeLetStatement(variable: NodeIdentifier, value: ASTNode, exported: boolean, location: LocationRange): NodeLetStatement;
export declare function nodeDefunStatement(variable: NodeIdentifier, value: NamedNodeLambda, exported: boolean, location: LocationRange): NodeDefunStatement;
export declare function nodeString(value: string, location: LocationRange): NodeString;
export declare function nodeTernary(condition: ASTNode, trueExpression: ASTNode, falseExpression: ASTNode, kind: NodeTernary["kind"], location: LocationRange): NodeTernary;
export declare function nodeVoid(location: LocationRange): NodeVoid;
export type ASTCommentNode = {
    type: "lineComment" | "blockComment";
    value: string;
    location: LocationRange;
};
export declare function lineComment(text: string, location: LocationRange): ASTCommentNode;
export declare function blockComment(text: string, location: LocationRange): ASTCommentNode;
export declare function parseEscapeSequence(char: string[], location: LocationRange, error: (e: any, l: LocationRange) => void): any;
export {};
//# sourceMappingURL=peggyHelpers.d.ts.map