import { BaseDist } from "./BaseDist.js";
import * as Result from "../utility/result.js";
import * as Operation from "../operation.js";
import { PointSetDist } from "./PointSetDist.js";
import { result } from "../utility/result.js";
import { DistError } from "./DistError.js";
import { OperationError } from "../operationError.js";
import { Env } from "./env.js";
type PointsetXSelection = "Linear" | "ByWeight";
export declare abstract class SymbolicDist extends BaseDist {
    private static minCdfValue;
    private static maxCdfValue;
    abstract toString(): string;
    toSparkline(bucketCount: number, env: Env): Result.result<string, DistError>;
    normalize(): this;
    integralSum(): number;
    protected abstract simplePdf(f: number): number;
    pdf(f: number): Result.result<number, DistError>;
    protected interpolateXs(opts: {
        xSelection: PointsetXSelection;
        points: number;
        env: Env;
    }): number[];
    toPointSetDist(env: Env, xSelection?: PointsetXSelection): result<PointSetDist, DistError>;
    truncate(left: number | undefined, right: number | undefined, opts?: {
        env: Env;
    }): result<BaseDist, DistError>;
    min(): number;
    max(): number;
    sampleN(n: number): number[];
    expectedConvolutionCost(): number;
    isFloat(): boolean;
}
export declare class Normal extends SymbolicDist {
    private _mean;
    private _stdev;
    private constructor();
    static make({ mean, stdev, }: {
        mean: number;
        stdev: number;
    }): result<Normal, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(x: number): number;
    sample(): number;
    mean(): number;
    stdev(): Result.result<number, DistError>;
    variance(): Result.result<number, DistError>;
    _isEqual(other: Normal): boolean;
    static fromCredibleInterval({ low, high, probability, }: {
        low: number;
        high: number;
        probability: number;
    }): result<Normal, string>;
    static add(n1: Normal, n2: Normal): Normal;
    static subtract(n1: Normal, n2: Normal): Normal;
    static operate(operation: Operation.AlgebraicOperation, n1: Normal, n2: Normal): Normal | undefined;
    static operateFloatFirst(operation: Operation.AlgebraicOperation, n1: number, n2: Normal): SymbolicDist | undefined;
    static operateFloatSecond(operation: Operation.AlgebraicOperation, n1: Normal, n2: number): SymbolicDist | undefined;
}
export declare class Exponential extends SymbolicDist {
    rate: number;
    private constructor();
    static make(rate: number): result<Exponential, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(p: number): number;
    sample(): number;
    mean(): number;
    variance(): result<number, DistError>;
    _isEqual(other: Exponential): boolean;
}
export declare class Cauchy extends SymbolicDist {
    local: number;
    scale: number;
    private constructor();
    static make({ local, scale, }: {
        local: number;
        scale: number;
    }): result<Cauchy, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(p: number): number;
    sample(): number;
    mean(): number;
    stdev(): result<number, DistError>;
    variance(): result<number, DistError>;
    _isEqual(other: Cauchy): boolean;
}
export declare class Triangular extends SymbolicDist {
    low: number;
    medium: number;
    high: number;
    private constructor();
    static make({ low, medium, high, }: {
        low: number;
        medium: number;
        high: number;
    }): result<Triangular, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(p: number): number;
    sample(): number;
    mean(): number;
    variance(): result<number, DistError>;
    _isEqual(other: Triangular): boolean;
    min(): number;
    max(): number;
}
export declare class Beta extends SymbolicDist {
    alpha: number;
    beta: number;
    private constructor();
    static make({ alpha, beta, }: {
        alpha: number;
        beta: number;
    }): result<Beta, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(x: number): number;
    sample(): number;
    mean(): number;
    variance(): result<number, DistError>;
    _isEqual(other: Beta): boolean;
    static fromMeanAndSampleSize({ mean, sampleSize, }: {
        mean: number;
        sampleSize: number;
    }): result<Beta, string>;
    static fromMeanAndStdev({ mean, stdev, }: {
        mean: number;
        stdev: number;
    }): result<Beta, string>;
}
export declare class Lognormal extends SymbolicDist {
    mu: number;
    sigma: number;
    private constructor();
    static make({ mu, sigma, }: {
        mu: number;
        sigma: number;
    }): result<Lognormal, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(x: number): number;
    sample(): number;
    mean(): number;
    variance(): Result.result<number, DistError>;
    _isEqual(other: Lognormal): boolean;
    static fromCredibleInterval({ low, high, probability, }: {
        low: number;
        high: number;
        probability: number;
    }): result<Lognormal, string>;
    static fromMeanAndStdev({ mean, stdev, }: {
        mean: number;
        stdev: number;
    }): result<Lognormal, string>;
    static multiply(l1: Lognormal, l2: Lognormal): Lognormal;
    static divide(l1: Lognormal, l2: Lognormal): Lognormal;
    static operate(operation: Operation.AlgebraicOperation, n1: Lognormal, n2: Lognormal): Lognormal | undefined;
    static operateFloatFirst(operation: Operation.AlgebraicOperation, n1: number, n2: Lognormal): Lognormal | undefined;
    static operateFloatSecond(operation: Operation.AlgebraicOperation, n1: Lognormal, n2: number): Lognormal | undefined;
}
export declare class Uniform extends SymbolicDist {
    low: number;
    high: number;
    private constructor();
    static make({ low, high, }: {
        low: number;
        high: number;
    }): result<Uniform, string>;
    protected interpolateXs(opts: {
        xSelection: PointsetXSelection;
        points: number;
        env: Env;
    }): number[];
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(x: number): number;
    sample(): number;
    mean(): number;
    variance(): result<number, DistError>;
    _isEqual(other: Uniform): boolean;
    min(): number;
    max(): number;
    truncate(left: number | undefined, right: number | undefined): result<Uniform, DistError>;
}
export declare class Logistic extends SymbolicDist {
    location: number;
    scale: number;
    private constructor();
    static make({ location, scale, }: {
        location: number;
        scale: number;
    }): result<Logistic, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(p: number): number;
    sample(): number;
    mean(): number;
    variance(): Result.result<number, DistError>;
    _isEqual(other: Logistic): boolean;
}
export declare class Bernoulli extends SymbolicDist {
    p: number;
    private constructor();
    static make(p: number): result<Bernoulli, string>;
    toString(): string;
    private pmf;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(prob: number): 1 | 0;
    mean(): number;
    sample(): 1 | 0;
    _isEqual(other: Bernoulli): boolean;
    min(): 1 | 0;
    max(): 1 | 0;
    variance(): Result.result<number, DistError>;
    toPointSetDist(): result<PointSetDist, DistError>;
}
export declare class Gamma extends SymbolicDist {
    shape: number;
    scale: number;
    private constructor();
    static make({ shape, scale, }: {
        shape: number;
        scale: number;
    }): result<Gamma, string>;
    toString(): string;
    simplePdf(x: number): number;
    cdf(x: number): number;
    inv(x: number): number;
    sample(): number;
    mean(): number;
    variance(): Result.result<number, DistError>;
    _isEqual(other: Gamma): boolean;
}
export declare class PointMass extends SymbolicDist {
    t: number;
    constructor(t: number);
    toString(): string;
    static make(t: number): result<PointMass, string>;
    simplePdf(x: number): 1 | 0;
    cdf(x: number): 1 | 0;
    inv(p: number): number;
    mean(): number;
    variance(): result<number, DistError>;
    sample(): number;
    _isEqual(other: PointMass): boolean;
    min(): number;
    max(): number;
    expectedConvolutionCost(): number;
    isFloat(): boolean;
    toPointSetDist(): result<PointSetDist, DistError>;
}
export declare class Binomial extends SymbolicDist {
    n: number;
    p: number;
    constructor(n: number, p: number);
    toString(): string;
    static make(n: number, p: number): result<Binomial, string>;
    simplePdf(x: number): number;
    cdf(k: number): number;
    inv(p: number): number;
    mean(): number;
    variance(): result<number, DistError>;
    sample(): number;
    _isEqual(other: Binomial): boolean;
    toPointSetDist(): result<PointSetDist, DistError>;
}
export declare class Poisson extends SymbolicDist {
    lambda: number;
    constructor(lambda: number);
    toString(): string;
    static make(lambda: number): result<Poisson, string>;
    simplePdf(x: number): number;
    cdf(k: number): number;
    inv(p: number): number;
    mean(): number;
    variance(): result<number, DistError>;
    sample(): number;
    _isEqual(other: Poisson): boolean;
    toPointSetDist(): result<PointSetDist, DistError>;
}
export declare const tryAnalyticalSimplification: (d1: SymbolicDist, d2: SymbolicDist, op: Operation.AlgebraicOperation) => result<SymbolicDist, OperationError> | undefined;
export {};
//# sourceMappingURL=SymbolicDist.d.ts.map