import * as magicNumbers from "../../magicNumbers.js";
import * as Operation from "../../operation.js";
import * as Result from "../../utility/result.js";
import { logarithmOfDistributionError, operationDistError, unreachableError, } from "../DistError.js";
import * as PointSetDist from "../PointSetDist.js";
import * as SampleSetDist from "../SampleSetDist/index.js";
import * as SymbolicDist from "../SymbolicDist.js";
const validateInputs = (t1, t2, arithmeticOperation) => {
    const getLogarithmInputError = () => {
        const isDistNotGreaterThanZero = (t) => t.cdf(magicNumbers.Epsilon.ten) > 0;
        if (isDistNotGreaterThanZero(t1)) {
            return logarithmOfDistributionError("First input must be completely greater than 0");
        }
        if (isDistNotGreaterThanZero(t2)) {
            return logarithmOfDistributionError("Second input must be completely greater than 0");
        }
        return undefined;
    };
    if (arithmeticOperation == "Logarithm") {
        return getLogarithmInputError();
    }
    else {
        return undefined;
    }
};
const symbolicStrategy = ({ t1, t2, arithmeticOperation, }) => {
    if (t1 instanceof SymbolicDist.SymbolicDist &&
        t2 instanceof SymbolicDist.SymbolicDist) {
        const result = SymbolicDist.tryAnalyticalSimplification(t1, t2, arithmeticOperation);
        return result ? Result.errMap(result, operationDistError) : undefined;
    }
    else {
        return undefined;
    }
};
const convolutionStrategy = ({ env, arithmeticOperation, t1, t2, }) => {
    const convOp = Operation.Convolution.fromAlgebraicOperation(arithmeticOperation);
    if (convOp === undefined) {
        return undefined;
    }
    const p1r = t1.toPointSetDist(env);
    const p2r = t2.toPointSetDist(env);
    if (!p1r.ok) {
        return p1r;
    }
    if (!p2r.ok) {
        return p2r;
    }
    const p1 = p1r.value;
    const p2 = p2r.value;
    return Result.Ok(PointSetDist.combineAlgebraically(convOp, p1, p2));
};
const monteCarloStrategy = ({ env, arithmeticOperation, t1, t2, }) => {
    const fn = Operation.Algebraic.toFn(arithmeticOperation);
    const s1r = SampleSetDist.SampleSetDist.fromDist(t1, env);
    const s2r = SampleSetDist.SampleSetDist.fromDist(t2, env);
    if (!s1r.ok) {
        return s1r;
    }
    if (!s2r.ok) {
        return s2r;
    }
    const s1 = s1r.value;
    const s2 = s2r.value;
    return SampleSetDist.map2({ fn, t1: s1, t2: s2 });
};
const preferConvolutionToMonteCarlo = (args) => {
    const hasSampleSetDist = () => args.t1 instanceof SampleSetDist.SampleSetDist ||
        args.t2 instanceof SampleSetDist.SampleSetDist;
    const convolutionIsFasterThanMonteCarlo = () => args.t1.expectedConvolutionCost() * args.t2.expectedConvolutionCost() <
        args.env.sampleCount;
    return (!hasSampleSetDist() &&
        Operation.Convolution.canDoAlgebraicOperation(args.arithmeticOperation) &&
        convolutionIsFasterThanMonteCarlo());
};
export const algebraicCombination = (args) => {
    const invalidOperationError = validateInputs(args.t1, args.t2, args.arithmeticOperation);
    if (invalidOperationError !== undefined) {
        return Result.Err(invalidOperationError);
    }
    const maybeSymbolicResult = symbolicStrategy(args);
    if (maybeSymbolicResult) {
        return maybeSymbolicResult;
    }
    const strategy = preferConvolutionToMonteCarlo(args)
        ? convolutionStrategy
        : monteCarloStrategy;
    return strategy(args) ?? Result.Err(unreachableError());
};
//# sourceMappingURL=algebraicCombination.js.map