import { BaseDist } from "../dist/BaseDist.js";
import { REOther } from "../errors/messages.js";
import { Lambda } from "../reducer/lambda.js";
import { ImmutableMap } from "../utility/immutableMap.js";
import { Domain } from "./domain.js";
export type ValueMap = ImmutableMap<string, Value>;
type Indexable = {
    get(key: Value): Value;
};
declare abstract class BaseValue {
    abstract type: string;
    abstract publicName: string;
    clone(): any;
    abstract toString(): string;
}
declare class VArray extends BaseValue implements Indexable {
    value: Value[];
    readonly type = "Array";
    readonly publicName = "List";
    constructor(value: Value[]);
    toString(): string;
    get(key: Value): Value;
    flatten(): VArray;
    shuffle(): VArray;
    isEqual(other: VArray): boolean;
}
export declare const vArray: (v: Value[]) => VArray;
declare class VBool extends BaseValue {
    value: boolean;
    readonly type = "Bool";
    readonly publicName = "Boolean";
    constructor(value: boolean);
    toString(): string;
    isEqual(other: VBool): boolean;
}
export declare const vBool: (v: boolean) => VBool;
declare class VDate extends BaseValue {
    value: Date;
    readonly type = "Date";
    readonly publicName = "Date";
    constructor(value: Date);
    toString(): string;
    isEqual(other: VDate): boolean;
}
export declare const vDate: (v: Date) => VDate;
declare class VDist extends BaseValue {
    value: BaseDist;
    readonly type = "Dist";
    readonly publicName = "Distribution";
    constructor(value: BaseDist);
    toString(): string;
    isEqual(other: VDist): boolean;
}
export declare const vDist: (v: BaseDist) => VDist;
declare class VLambda extends BaseValue implements Indexable {
    value: Lambda;
    readonly type = "Lambda";
    readonly publicName = "Function";
    constructor(value: Lambda);
    toString(): string;
    get(key: Value): VArray;
}
export declare const vLambda: (v: Lambda) => VLambda;
declare class VNumber extends BaseValue {
    value: number;
    readonly type = "Number";
    readonly publicName = "Number";
    constructor(value: number);
    toString(): string;
    isEqual(other: VNumber): boolean;
}
export declare const vNumber: (v: number) => VNumber;
declare class VString extends BaseValue {
    value: string;
    readonly type = "String";
    readonly publicName = "String";
    constructor(value: string);
    toString(): string;
    isEqual(other: VString): boolean;
}
export declare const vString: (v: string) => VString;
declare class VDict extends BaseValue implements Indexable {
    value: ValueMap;
    readonly type = "Dict";
    readonly publicName = "Dictionary";
    constructor(value: ValueMap);
    toString(): string;
    get(key: Value): Value;
    isEqual(other: VDict): boolean;
}
export declare const vDict: (v: ValueMap) => VDict;
declare class VTimeDuration extends BaseValue {
    value: number;
    readonly type = "TimeDuration";
    readonly publicName = "Time Duration";
    constructor(value: number);
    toString(): string;
    isEqual(other: VTimeDuration): boolean;
}
export declare const vTimeDuration: (v: number) => VTimeDuration;
export type CommonScaleArgs = {
    min?: number;
    max?: number;
    tickFormat?: string;
    title?: string;
};
export type Scale = CommonScaleArgs & ({
    type: "linear";
} | {
    type: "log";
} | {
    type: "symlog";
    constant?: number;
} | {
    type: "power";
    exponent?: number;
});
export declare const SCALE_SYMLOG_DEFAULT_CONSTANT = 0.0001;
export declare const SCALE_POWER_DEFAULT_CONSTANT = 0.1;
declare class VScale extends BaseValue {
    value: Scale;
    readonly type = "Scale";
    readonly publicName = "Scale";
    constructor(value: Scale);
    toString(): string;
    isEqual(other: VScale): boolean;
}
export declare const vScale: (scale: Scale) => VScale;
export type CommonInputArgs = {
    name: string;
    description?: string;
};
export type Input = CommonInputArgs & ({
    type: "text";
    default?: string;
} | {
    type: "textArea";
    default?: string;
} | {
    type: "checkbox";
    default?: boolean;
} | {
    type: "select";
    default?: string;
    options: string[];
});
declare class VInput extends BaseValue {
    value: Input;
    readonly type = "Input";
    readonly publicName = "Input";
    constructor(value: Input);
    toString(): string;
    isEqual(other: VInput): boolean;
}
export declare const vInput: (input: Input) => VInput;
export type LabeledDistribution = {
    name?: string;
    distribution: BaseDist;
};
export type CommonPlotArgs = {
    title?: string;
};
export type Plot = CommonPlotArgs & ({
    type: "distributions";
    distributions: LabeledDistribution[];
    xScale: Scale;
    yScale: Scale;
    showSummary: boolean;
} | {
    type: "numericFn";
    fn: Lambda;
    xScale: Scale;
    yScale: Scale;
    points?: number;
} | {
    type: "distFn";
    fn: Lambda;
    xScale: Scale;
    yScale: Scale;
    distXScale: Scale;
    points?: number;
} | {
    type: "scatter";
    xDist: BaseDist;
    yDist: BaseDist;
    xScale: Scale;
    yScale: Scale;
} | {
    type: "relativeValues";
    fn: Lambda;
    ids: string[];
});
export type TableChart = {
    data: Value[];
    title?: string;
    columns: {
        fn: Lambda;
        name: string | undefined;
    }[];
};
declare class VTableChart extends BaseValue {
    value: TableChart;
    readonly type = "TableChart";
    readonly publicName = "Table Chart";
    constructor(value: TableChart);
    toString(): string;
}
export declare const vTableChart: (v: TableChart) => VTableChart;
export type Calculator = {
    fn: Lambda;
    inputs: Input[];
    autorun: boolean;
    description?: string;
    title?: string;
    sampleCount?: number;
};
declare class VCalculator extends BaseValue {
    value: Calculator;
    readonly type = "Calculator";
    readonly publicName = "Calculator";
    private error;
    constructor(value: Calculator);
    private setError;
    getError(): REOther | null;
    toString(): string;
}
export declare const vCalculator: (v: Calculator) => VCalculator;
declare class VPlot extends BaseValue implements Indexable {
    value: Plot;
    readonly type = "Plot";
    readonly publicName = "Plot";
    constructor(value: Plot);
    toString(): string;
    get(key: Value): VLambda;
}
export declare const vPlot: (plot: Plot) => VPlot;
export declare class VDomain extends BaseValue implements Indexable {
    value: Domain;
    readonly type = "Domain";
    readonly publicName = "Domain";
    constructor(value: Domain);
    toString(): string;
    get(key: Value): VNumber;
    isEqual(other: VDomain): boolean;
}
export declare const vDomain: (domain: Domain) => VDomain;
declare class VVoid extends BaseValue {
    readonly type = "Void";
    readonly publicName = "Void";
    constructor();
    toString(): string;
}
export declare const vVoid: () => VVoid;
export type Value = VArray | VBool | VDate | VDist | VLambda | VNumber | VString | VDict | VTimeDuration | VPlot | VTableChart | VCalculator | VScale | VInput | VDomain | VVoid;
export declare function isEqual(a: Value, b: Value): boolean;
export declare function uniq(array: Value[]): Value[];
export declare function uniqBy(array: Value[], fn: (e: Value) => Value): Value[];
export {};
//# sourceMappingURL=index.d.ts.map